@@ -1,5 +1,3 @@
-utils::globalVariables(c("Model", "Value", "Measure"))
-
 #' Plot Methods for \code{fastml} Objects
 #'
 #' \code{plot.fastml} produces visual diagnostics for a trained \code{fastml} object.
@@ -13,6 +11,7 @@ utils::globalVariables(c("Model", "Value", "Measure"))
 #'     \item{\code{"roc"}}{ROC curve(s) for binary classification models.}
 #'     \item{\code{"calibration"}}{Calibration plot for the best model(s).}
 #'     \item{\code{"residual"}}{Residual diagnostics for the best model.}
+#'     \item{\code{"learning_curve"}}{Learning-curve plot if recorded during training.}
 #'     \item{\code{"all"}}{Produce all available plots.}
 #'   }
 #' @param ... Additional arguments (currently unused).
@@ -58,7 +57,7 @@ utils::globalVariables(c("Model", "Value", "Measure"))
 #' @export
 plot.fastml <- function(x,
                         algorithm = "best",
-                        type = c("all", "bar", "roc", "calibration", "residual"),
+                        type = c("all", "bar", "roc", "calibration", "residual", "learning_curve"),
                         ...) {
 
   if (!inherits(x, "fastml")) {
@@ -68,7 +67,7 @@ plot.fastml <- function(x,
   # Validate 'type' argument
   type <- match.arg(type, several.ok = TRUE)
   if ("all" %in% type) {
-    type <- c("bar", "roc", "calibration", "residual")
+    type <- c("bar", "roc", "calibration", "residual", "learning_curve")
   }
 
   performance      <- x$performance
@@ -78,6 +77,8 @@ plot.fastml <- function(x,
   optimized_metric <- x$metric
   positive_class   <- x$positive_class
   engine_names     <- x$engine_names
+  resampling_plan  <- x$resampling_plan
+  resampling_desc  <- fastml_describe_resampling(resampling_plan)
 
   # Rebuild performance_wide (robust to single- or multi-engine structures)
   metrics_list <- lapply(names(performance), function(model_name) {
@@ -141,8 +142,8 @@ plot.fastml <- function(x,
   if (length(engine_names) == 1 && "LiblineaR" %in% engine_names) {
     performance_wide <- tidyr::pivot_wider(
       performance_sub,
-      names_from  = .metric,
-      values_from = .estimate
+      names_from  = .data$.metric,
+      values_from = .data$.estimate
     ) %>%
       dplyr::select(Model, Engine, accuracy, kap, sens, spec, precision, f_meas)
   } else {
@@ -152,8 +153,8 @@ plot.fastml <- function(x,
     }
     performance_wide <- tidyr::pivot_wider(
       performance_sub,
-      names_from  = .metric,
-      values_from = .estimate
+      names_from  = .data$.metric,
+      values_from = .data$.estimate
     )
     select_cols <- c("Model", "Engine", keep_metrics)
     select_cols <- intersect(select_cols, colnames(performance_wide))
@@ -251,7 +252,7 @@ plot.fastml <- function(x,
 
     p_bar <- ggplot2::ggplot(
       performance_melt,
-      ggplot2::aes(x = Model, y = Value, fill = Engine)
+      ggplot2::aes(x = .data$Model, y = .data$Value, fill = .data$Engine)
     ) +
       ggplot2::geom_bar(stat = "identity", position = ggplot2::position_dodge()) +
       ggplot2::facet_wrap(~ Metric, scales = "free_y") +
@@ -396,10 +397,14 @@ plot.fastml <- function(x,
 
 
           # Create the ROC curve plot, using the compound ModelEngine label for color
-          roc_curve_plot <- ggplot(data = roc_data,
-                                   aes(x = 1 - FalsePositiveRate,
-                                       y = TruePositiveRate,
-                                       color = ModelEngine)) +
+          roc_curve_plot <- ggplot(
+            data = roc_data,
+            aes(
+              x = 1 - .data$FalsePositiveRate,
+              y = .data$TruePositiveRate,
+              color = .data$ModelEngine
+            )
+          ) +
             geom_line(linewidth = 1) +
             geom_abline(intercept = 0, slope = 1, linetype = "dashed", color = "grey") +
             theme_minimal() +
@@ -466,7 +471,9 @@ plot.fastml <- function(x,
               estimate = !!rlang::sym(pred_col),
               event_level = x$event_class
             ) +
-              ggplot2::labs(title = paste("Calibration Plot for", model_name))
+              ggplot2::labs(
+                title = paste("Calibration Plot for", model_name)
+              )
             print(p_cal)
           }
         } else {
@@ -502,20 +509,34 @@ plot.fastml <- function(x,
       names_df_best <- unique(unlist(lapply(df_best, names)))
       if (!is.null(df_best) && "truth" %in% names_df_best && "estimate" %in% names_df_best) {
         df_best_all <- dplyr::bind_rows(df_best, .id = "ModelEngine") %>%
-          dplyr::mutate(residual = truth - estimate)
+          dplyr::mutate(residual = .data$truth - .data$estimate)
 
         cat("\nResidual Diagnostics for Best Model:\n")
 
-        p_truth_pred <- ggplot2::ggplot(df_best_all, ggplot2::aes(x = estimate, y = truth)) +
+        p_truth_pred <- ggplot2::ggplot(
+          df_best_all,
+          ggplot2::aes(x = .data$estimate, y = .data$truth)
+        ) +
           ggplot2::geom_point(alpha = 0.6) +
           ggplot2::geom_abline(linetype = "dashed", color = "red") +
-          ggplot2::labs(title = "Truth vs Predicted", x = "Predicted", y = "Truth") +
+          ggplot2::labs(
+            title = "Truth vs Predicted",
+            x = "Predicted",
+            y = "Truth"
+          ) +
           ggplot2::theme_bw()
         print(p_truth_pred)
 
-        p_resid_hist <- ggplot2::ggplot(df_best_all, ggplot2::aes(x = residual)) +
+        p_resid_hist <- ggplot2::ggplot(
+          df_best_all,
+          ggplot2::aes(x = .data$residual)
+        ) +
           ggplot2::geom_histogram(bins = 30, fill = "steelblue", color = "white", alpha = 0.7) +
-          ggplot2::labs(title = "Residual Distribution", x = "Residual", y = "Count") +
+          ggplot2::labs(
+            title = "Residual Distribution",
+            x = "Residual",
+            y = "Count"
+          ) +
           ggplot2::theme_bw()
         print(p_resid_hist)
       } else {
@@ -526,5 +547,29 @@ plot.fastml <- function(x,
     }
   }
 
+  # 5. Learning curve (optional)
+  if ("learning_curve" %in% type) {
+    lc <- x$learning_curve
+    if (!is.null(lc) && !is.null(lc$plot)) {
+      print(lc$plot)
+    } else if (!is.null(lc) && !is.null(lc$data)) {
+      lc_plot <- ggplot2::ggplot(
+        lc$data,
+        ggplot2::aes(x = .data$Fraction, y = .data$Performance)
+      ) +
+        ggplot2::geom_line(color = "blue") +
+        ggplot2::geom_point(color = "blue") +
+        ggplot2::labs(
+          title = "Learning Curve",
+          x = "Training Set Size (fraction)",
+          y = paste("Mean", optimized_metric, "across models")
+        ) +
+        ggplot2::theme_minimal()
+      print(lc_plot)
+    } else {
+      cat("\nNo learning curve data available. Set `learning_curve = TRUE` when fitting to record it.\n\n")
+    }
+  }
+
   invisible(x)
 }
