@@ -4,6 +4,7 @@ library(dplyr)
 skip_if_not_installed("lime")
 skip_if_not_installed("pdp")
 skip_if_not_installed("iml")
+skip_if_not_installed("ceterisParibus")
 
 # small dataset for tests
 set.seed(123)
@@ -20,19 +21,14 @@ model <- fastml(
 )
 
 
-test_that("LIME explanation works", {
-  expl <- fastexplain(model, method = "lime")
-  expect_s3_class(expl, "data.frame")
+
+test_that("ALE explanation runs", {
+  expect_error(
+    fastexplain(model, method = "ale", features = "Sepal.Length"),
+    regexp = NA
+  )
 })
 
-# test_that("ICE plot runs", {
-#   expect_silent(fastexplain(model, method = "ice", features = "Sepal.Length"))
-# })
-
-# test_that("ALE explanation runs", {
-#   expect_silent(fastexplain(model, method = "ale", features = "Sepal.Length"))
-# })
-
 test_that("DALEX explanation runs cleanly", {
   expect_error(
     suppressMessages(suppressWarnings(fastexplain(model))),
@@ -40,3 +36,8 @@ test_that("DALEX explanation runs cleanly", {
   )
 })
 
+test_that("Counterfactual explanation computes counterfactuals", {
+  cf <- suppressWarnings(fastexplain(model, method = "counterfactual", observation = iris[1, ]))
+  expect_true(is.list(cf))
+})
+
